package hu.bme.mit.ftsrg.healthcentre.session;

import hu.bme.mit.ftsrg.healthcentre.StreamEOFException;
import hu.bme.mit.ftsrg.healthcentre.model.HealthCentreModel;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Arrays;
import java.util.SortedMap;
import java.util.TreeMap;

public abstract class CommandLine {
	protected final BufferedReader in;
	protected final PrintStream out;
	protected final HealthCentreModel model;

	protected SortedMap<String, Command> commands = new TreeMap<>();

	public CommandLine(HealthCentreModel model, BufferedReader in,
			PrintStream out) {
		this.in = in;
		this.out = out;
		this.model = model;
	}

	public final void run() throws IOException, StreamEOFException {
		// print help
		help();

		boolean shouldExit = false;
		while (!shouldExit) {
			// read command
			out.print("Enter command: ");
			out.flush();
			String commandString = in.readLine();
			if (commandString == null)
				throw new StreamEOFException();

			// parse command
			String[] parts = commandString.trim().split("\\s+");
			String commandName = parts[0];

			// execute command
			switch (commandName) {
			case "help":
				help();
				break;

			case "exit":
				shouldExit = true;
				break;

			default:
				// get Command object
				Command command = commands.get(commandName);

				if (command != null) {
					// valid command, create args
					String[] args;

					if (parts.length > 1) {
						args = Arrays.copyOfRange(parts, 1, parts.length);
					} else {
						args = new String[0];
					}

					// we should avoid conflicts
					synchronized (model) {
						// execute
						command.execute(args);
					}
				} else {
					out.println("Unknown command");
				}
				break;
			}

		}
	}

	private final void help() {
		out.println("Available commands:");
		for (Command command : commands.values()) {
			out.println("  " + command.getUsage());
		}
	}

	protected abstract class Command {
		public final void execute(String[] args) {
			if (validateArgs(args)) {
				executeCommand(args);
			} else {
				out.println(getUsage());
			}
		}

		protected abstract void executeCommand(String[] args);

		protected abstract boolean validateArgs(String[] args);

		public abstract String getUsage();
	}
}
