package hu.bme.mit.ftsrg.healthcentre.session;

import hu.bme.mit.ftsrg.healthcentre.HealthCentreApplication;
import hu.bme.mit.ftsrg.healthcentre.model.Appointment;
import hu.bme.mit.ftsrg.healthcentre.model.Doctor;
import hu.bme.mit.ftsrg.healthcentre.model.HealthCentreModel;

import java.io.BufferedReader;
import java.io.PrintStream;
import java.text.ParseException;
import java.util.Date;

public final class DoctorCommandLine extends CommandLine {
	private final Doctor doctor;

	public DoctorCommandLine(Doctor doctor, HealthCentreModel model,
			BufferedReader in, PrintStream out) {
		super(model, in, out);
		this.doctor = doctor;

		commands.put("me", new MeCommand());
		commands.put("list", new ListCommand());
		commands.put("create", new CreateCommand());
		commands.put("delete", new DeleteCommand());
	}

	private final class MeCommand extends Command {
		@Override
		protected void executeCommand(String[] args) {
			out.println(doctor);
		}

		@Override
		protected boolean validateArgs(String[] args) {
			return true;
		}

		@Override
		public String getUsage() {
			return "Usage: me";
		}
	}

	private final class ListCommand extends Command {
		@Override
		protected void executeCommand(String[] args) {
			switch (args[0]) {
			case "appointments":
				// list appointments
				out.println("Appointments:");
				for (Appointment appointment : doctor.appointments()) {
					out.println(String.format("  %s: %s",
							HealthCentreApplication.DATE_FORMAT
									.format(appointment.getDate()), appointment
									.getPatient()));
				}
				break;

			case "availabledates":
				// list available dates
				out.println("Available dates:");
				for (Date availableDate : doctor.availableDates()) {
					out.println(String.format("  %s",
							HealthCentreApplication.DATE_FORMAT
									.format(availableDate)));
				}
				break;
			}
		}

		@Override
		protected boolean validateArgs(String[] args) {
			return args.length == 1
					&& ("appointments".equals(args[0]) || "availabledates"
							.equals(args[0]));
		}

		@Override
		public String getUsage() {
			return "Usage: list appointments or list availabledates";
		}
	}

	private final class CreateCommand extends Command {
		@Override
		protected void executeCommand(String[] args) {
			try {
				// parse date
				Date date = HealthCentreApplication.DATE_FORMAT.parse(args[0]
						+ ' ' + args[1]);
				
				// check whether it already exists
				if (doctor.availableDates().contains(date)) {
					out.println("The specified date already exists");
				} else {
					// check whether there is an appointment at this date
					if (doctor.getAppointmentByDate(date) == null) {
						// save
						doctor.availableDates().add(date);
						out.println("Done");
					} else {
						out.println("You already have an appointment on this date");
					}

				}
			} catch (ParseException e) {
				out.println("Bad date format");
			}
		}

		@Override
		protected boolean validateArgs(String[] args) {
			return args.length == 2;
		}

		@Override
		public String getUsage() {
			return "Usage: create <available date in format "
					+ HealthCentreApplication.DATE_FORMAT_STRING + ">";
		}
	}

	private final class DeleteCommand extends Command {
		@Override
		protected void executeCommand(String[] args) {
			try {
				// parse date
				Date date = HealthCentreApplication.DATE_FORMAT.parse(args[0]
						+ ' ' + args[1]);
				
				// remove
				if (doctor.availableDates().remove(date)) {
					out.println("Done");
				} else {
					out.println("The specified date was not found");
				}
			} catch (ParseException e) {
				out.println("Bad date format");
			}
		}

		@Override
		protected boolean validateArgs(String[] args) {
			return args.length == 2;
		}

		@Override
		public String getUsage() {
			return "Usage: delete <available date in format "
					+ HealthCentreApplication.DATE_FORMAT_STRING + ">";
		}
	}
}
