package hu.bme.mit.ftsrg.healthcentre.session;

import hu.bme.mit.ftsrg.healthcentre.HealthCentreApplication;
import hu.bme.mit.ftsrg.healthcentre.model.Appointment;
import hu.bme.mit.ftsrg.healthcentre.model.Doctor;
import hu.bme.mit.ftsrg.healthcentre.model.HealthCentreModel;
import hu.bme.mit.ftsrg.healthcentre.model.Patient;

import java.io.BufferedReader;
import java.io.PrintStream;
import java.text.ParseException;
import java.util.Date;

public final class PatientCommandLine extends CommandLine {
	private final Patient patient;

	public PatientCommandLine(Patient patient, HealthCentreModel model,
			BufferedReader in, PrintStream out) {
		super(model, in, out);
		this.patient = patient;

		commands.put("me", new MeCommand());
		commands.put("list", new ListCommand());
		commands.put("create", new CreateCommand());
		commands.put("delete", new DeleteCommand());
	}

	private final class MeCommand extends Command {
		@Override
		protected void executeCommand(String[] args) {
			out.println(patient);
		}

		@Override
		protected boolean validateArgs(String[] args) {
			return true;
		}

		@Override
		public String getUsage() {
			return "Usage: me";
		}
	}

	private final class ListCommand extends Command {
		@Override
		protected void executeCommand(String[] args) {
			switch (args[0]) {
			case "appointments":
				// list appointments
				out.println("Appointments:");
				for (Appointment appointment : patient.appointments()) {
					out.println(String.format("  %s: %s",
							HealthCentreApplication.DATE_FORMAT
									.format(appointment.getDate()), appointment
									.getDoctor()));
				}
				break;

			case "doctors":
				// list doctors
				out.println("Doctors:");
				for (Doctor doctor : model.doctors().values()) {
					out.println("  " + doctor);
				}
				break;

			case "availabledates":
				// list available dates for a doctor

				// get Doctor object
				Doctor doctor = null;
				try {
					int sealNumber = Integer.parseInt(args[1]);
					for (Doctor d : model.doctors().values()) {
						if (d.getSealNumber() == sealNumber) {
							doctor = d;
							break;
						}
					}
				} catch (NumberFormatException e) {
				}

				if (doctor != null) {
					// list dates
					out.println("Available dates:");
					for (Date availableDate : doctor.availableDates()) {
						out.println(String.format("  %s",
								HealthCentreApplication.DATE_FORMAT
										.format(availableDate)));
					}
				} else {
					out.println("Please enter a valid seal number");
				}
				break;
			}
		}

		@Override
		protected boolean validateArgs(String[] args) {
			return (args.length == 1 && ("appointments".equals(args[0]) || "doctors"
					.equals(args[0])))
					|| args.length == 2
					&& "availabledates".equals(args[0]);
		}

		@Override
		public String getUsage() {
			return "Usage: list appointments or list doctors or list availabledates <seal number>";
		}
	}

	private final class CreateCommand extends Command {
		@Override
		protected void executeCommand(String[] args) {
			// get doctor object
			Doctor doctor = null;
			try {
				int sealNumber = Integer.parseInt(args[0]);
				for (Doctor d : model.doctors().values()) {
					if (d.getSealNumber() == sealNumber) {
						doctor = d;
						break;
					}
				}
			} catch (NumberFormatException e) {
			}

			if (doctor != null) {
				try {
					// parse date
					Date date = HealthCentreApplication.DATE_FORMAT
							.parse(args[1] + ' ' + args[2]);

					// check whether there is no appointment on this date
					if (patient.getAppointmentByDate(date) == null) {
						if (doctor.availableDates().contains(date)) {
							// delete
							doctor.availableDates().remove(date);
							new Appointment(doctor, patient, date);
							out.println("Done");
						} else {
							out.println("The specified date was not found");
						}
					} else {
						out.println("You already have an appointment on this date");
					}
				} catch (ParseException e) {
					out.println("Bad date format");
				}
			} else {
				out.println("Please enter a valid seal number");
			}
		}

		@Override
		protected boolean validateArgs(String[] args) {
			return args.length == 3;
		}

		@Override
		public String getUsage() {
			return "Usage: create <seal number> <available date in format "
					+ HealthCentreApplication.DATE_FORMAT_STRING + ">";
		}
	}

	private final class DeleteCommand extends Command {
		@Override
		protected void executeCommand(String[] args) {
			try {
				// parse date
				Date date = HealthCentreApplication.DATE_FORMAT.parse(args[0]
						+ ' ' + args[1]);

				// get Appointment object
				Appointment appointment = patient.getAppointmentByDate(date);

				if (appointment != null) {
					// delete
					appointment.delete();
					out.println("Done");
				} else {
					out.println("The specified appointment was not found");
				}
			} catch (ParseException e) {
				out.println("Bad date format");
			}
		}

		@Override
		protected boolean validateArgs(String[] args) {
			return args.length == 2;
		}

		@Override
		public String getUsage() {
			return "Usage: delete <appointment date in format "
					+ HealthCentreApplication.DATE_FORMAT_STRING + ">";
		}
	}
}
