package hu.bme.mit.ftsrg.healthcentre.session;

import hu.bme.mit.ftsrg.healthcentre.StreamEOFException;
import hu.bme.mit.ftsrg.healthcentre.model.Doctor;
import hu.bme.mit.ftsrg.healthcentre.model.HealthCentreModel;
import hu.bme.mit.ftsrg.healthcentre.model.Patient;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.Socket;

public class Session implements Runnable {
	private final Socket socket;
	private final HealthCentreModel model;
	private final PrintStream serverOut;
	private final PrintStream serverErr;
	private final BufferedReader socketIn;
	private final PrintStream socketOut;

	public Session(Socket socket, HealthCentreModel model,
			PrintStream serverOut, PrintStream serverErr) throws IOException {
		this.socket = socket;
		this.model = model;
		this.serverOut = serverOut;
		this.serverErr = serverErr;
		this.socketIn = new BufferedReader(new InputStreamReader(
				socket.getInputStream()));
		this.socketOut = new PrintStream(socket.getOutputStream(), true);
	}

	@Override
	public void run() {
		try {
			socketOut.println("Welcome to the Health Centre application");

			String name = prompt("Please enter your name: ");

			Doctor doctor;
			// we should avoid conflicts
			synchronized (model) {
				doctor = model.doctors().get(name);
			}

			// decide whether doctor or patient
			if (doctor != null) {
				// authenticate by password
				int passwordTrials = 0;
				while (true) {
					passwordTrials++;
					String password = prompt("Please enter your password: ");

					if (password.equals(doctor.getPassword())) {
						break;
					} else {
						if (passwordTrials > 3) {
							socketOut
									.println("Too many trials, disconnecting.");
							return;
						} else {
							socketOut.println("Password mismatch, try again.");
						}
					}
				}

				// start command line
				socketOut.println("Hello " + doctor);
				new DoctorCommandLine(doctor, model, socketIn, socketOut).run();
			} else {
				Patient patient;
				// we should avoid conflicts
				synchronized (model) {
					patient = model.patients().get(name);
				}

				// create new Patient object if it was not found
				if (patient == null) {
					String insuranceNumber = prompt("Please enter your insurance number: ");
					patient = new Patient(name, insuranceNumber);

					// we should avoid conflicts
					synchronized (model) {
						model.patients().put(name, patient);
					}
				}

				// start command line
				socketOut.println("Hello " + patient);
				new PatientCommandLine(patient, model, socketIn, socketOut)
						.run();
			}

			socketOut.print("Goodbye");
		} catch (IOException e) {
			// exception when accessing the socket
			serverErr
					.println("Unexpected exception occured when accessing the client: "
							+ socket.getInetAddress() + ":" + socket.getPort());
			e.printStackTrace(serverErr);
		} catch (StreamEOFException e) {
			// input EOF
			serverOut.println("Client EOF: " + socket.getInetAddress() + ":"
					+ socket.getPort());
		} finally {
			// close socket
			try {
				socket.close();
				serverOut.println("Client disconnected: "
						+ socket.getInetAddress() + ":" + socket.getPort());
			} catch (Exception e) {
				serverErr
						.println("Unexpected exception occured when closing the socket: "
								+ socket.getInetAddress()
								+ ":"
								+ socket.getPort());
				e.printStackTrace(serverErr);
			}
		}
	}

	public String prompt(String promptMessage) throws IOException,
			StreamEOFException {
		socketOut.print(promptMessage);
		socketOut.flush();
		return readLine();
	}

	public String readLine() throws IOException, StreamEOFException {
		String line = socketIn.readLine();
		if (line == null)
			throw new StreamEOFException();
		return line.trim();
	}
}
